#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 19 13:12:18 2019

@author: Benedict Wilkins
"""

import setuptools

setuptools.setup(name='pystarworlds',
      version='0.0.2',
      description='',
      url='https://github.com/dicelab-rhul/pystarworlds',
      author='Nausheen Saba, Benedict Wilkins, Joel Clarke',
      author_email='nausheen.shahid.2017@live.rhul.ac.uk, brjw@hotmail.co.uk, joel.clarke273@gmail.com',
      license='GNU General Public License v3 (GPLv3)',
      packages=setuptools.find_packages(),
      classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ])
